/*

  xml-HTB tools           

  Copyright (c) 2004, Claudiu Cismaru (claudiu at cnixs dot com)

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option) 
  any later version.
 
  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License   
  for more details.
  
  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc., 59
  Temple Place, Suite 330, Boston, MA 02111-1307 USA

*/

#include <stdio.h>
#include <stdlib.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include "globals.h"
#include "stack.h"

void usage (char *progname) {

  fprintf (stderr, "usage: \n  %s config_file.xml\n", progname);

  return;
}

int main (int argc, char **argv) {

  xmlDocPtr doc;
  xmlNodePtr node;
  
  char *configFile;
  
  xmlChar *attr;

  if (argc < 2) {
    usage (argv [0]);
    exit (EXIT_FAILURE);
  }

  configFile = (char *)strdup (argv [1]);

  xmlDoValidityCheckingDefaultValue = 1;

  doc = xmlParseFile (configFile);
  
  if (! doc) {
    DEBUG ("# Error parsing the document. Abort\n");
    exit (EXIT_FAILURE);
  }

  /*
    Don't need for checking if the document root is "htb"
    we assume it's correct cf. to DTD. If some admin changes
    the DTD then he/she does it on he/she own risk.
  */

  node = xmlDocGetRootElement (doc);

  /*
    Get the interfaces attributes from <htb> element.
    We also assume that "inif" exits in <htb> element, IF the DTD
    was not changed accidentaly, elsewere the program exits
    at parse time.
  */

  htbInif = htbOutif = NULL;

  attr = xmlGetProp (node, "if");
  htbInif = (char *) strdup (attr);
  xmlFree (attr);
  
  attr = xmlGetProp (node, "outif");

  if (attr != NULL)
    htbOutif = (char *) strdup (attr);  

  xmlFree (attr);

  attr = xmlGetProp (node, "leaf");

  htbClassLeaf = (char *) strdup (attr);

  xmlFree (attr);

  printf ("# HTB for interfaces\n");
  printf ("# in (OUR side): %s, out (PROVIDER side): %s\n", htbInif, htbOutif);

  attr = xmlGetProp (node, "debug");

  if (attr != NULL) {
    printf ("\n# WARNING: debug is ON !\nset -x\n\n");
  }

  xmlFree (attr);

  attr = xmlGetProp (node, "tc");

  printf ("TC=\"%s\"\n\n", attr);

  xmlFree (attr);

  printf ("DEV=\"%s\"\n", htbInif);

  if (htbOutif != NULL)
    printf ("OUTDEV=\"%s\"\n", htbOutif);

  printf ("\n");

  printf ("$TC qdisc del dev $DEV root >/dev/null 2>&1\n");
  printf ("$TC qdisc add dev $DEV root handle 0x01:0x00 htb default 0xffff\n");

  if (htbOutif == NULL)
    goto qdiscContinue;

  printf ("$TC qdisc del dev $OUTDEV root >/dev/null 2>&1\n");
  printf ("$TC qdisc add dev $OUTDEV root handle 0x01:0x00 htb default 0xffff\n");

qdiscContinue:

  node = node->children;

  lvlStackInit ();
  handleDoc (doc, node);

  free (configFile);
  free (htbInif);
  free (htbOutif);  
  free (htbClassLeaf);

  xmlFreeDoc (doc);

  exit (EXIT_SUCCESS);

}
