/*

  xml-HTB tools           

  Copyright (c) 2004, Claudiu Cismaru (claudiu at cnixs dot com)

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option) 
  any later version.
 
  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License   
  for more details.
  
  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc., 59
  Temple Place, Suite 330, Boston, MA 02111-1307 USA

*/

#include <stdio.h>
#include <libxml/parser.h>
#include "parse-doc.h"

void handleDoc (xmlDocPtr doc, xmlNodePtr cur) {

  static int depth = 0;
  static uint32_t lvlIdx = 1;

  uint32_t currentId, parentId;
  uint32_t useIf;

  xmlChar *attr;
  
  xmlChar *name, *rate, *ceil, *outrate, *outceil, *burst, *prio, *onlyif;

  xmlChar *type, *src, *dst, *sport, *dport, *mark;
  
  xmlChar *data, *leafData;

  if (cur->type == XML_ELEMENT_NODE) {


    /*

      Processing <default> element

    */

    if (! xmlStrcmp (cur->name, (xmlChar *)"default")) {

      currentId = 0xffff;

      /*
        I assume that the parser respects the DTD.
        This help me to get rid of some extra verification code
      */

      rate = xmlGetProp (cur, "rate");
      ceil = xmlGetProp (cur, "ceil");

      printf ("\n# add default class\n");
      printf ("$TC class add dev $DEV parent 0x01:0x00 classid 0x01:0x%04x htb rate %s ceil %s %s\n",
        currentId, rate, ceil, HTB_QUANTUM);

      if (htbOutif == NULL)
        goto defaultContinue;

      printf ("$TC class add dev $OUTDEV parent 0x01:0x00 classid 0x01:0x%04x htb rate %s ceil %s %s\n",
        currentId, rate, ceil, HTB_QUANTUM);

defaultContinue:

      xmlFree (rate);
      xmlFree (ceil);

    }

    /*

      Processing <class> element

    */

    if (! xmlStrcmp (cur->name, (xmlChar *)"class")) {

      uint32_t tmpVal;

      currentId = depth * CLS_PER_DEPTH + lvlIdx;

      tmpVal = 0;

      if (depth > 0) {
        lvlStackGet (depth - 1, &tmpVal);
        parentId = (depth - 1) * CLS_PER_DEPTH + tmpVal - 1;
      }
      else
        parentId = depth * CLS_PER_DEPTH + tmpVal;

      /* get class properties */

      rate = xmlGetProp (cur, "rate");
      ceil = xmlGetProp (cur, "ceil");

      if (ceil == NULL)
        ceil = (xmlChar *) strdup (rate);

      outrate = xmlGetProp (cur, "outrate");
      outceil = xmlGetProp (cur, "outceil");
      
      if (outrate == NULL)
        outrate = (xmlChar *) strdup (rate);
      if (outceil == NULL)
        outceil = (xmlChar *) strdup (ceil);

      name = xmlGetProp (cur, "name");
      burst = xmlGetProp (cur, "burst");
      prio = xmlGetProp (cur, "prio");

      onlyif = xmlGetProp (cur, "onlyif");

      useIf = XML_HTB_IFALL;

      if (onlyif != NULL) {
        if (! xmlStrcmp (onlyif, "in")) {
          useIf &= XML_HTB_IFIN;
          goto classIfCheckDone;
        }

        if (! xmlStrcmp (onlyif, "out"))
          useIf &= XML_HTB_IFOUT;
      }

classIfCheckDone:

      printf ("\n# add class, name: %s\n", name);

      if ((useIf & XML_HTB_IFIN) > 0) {
        printf ("$TC class add dev $DEV parent 0x01:0x%04x classid 0x01:0x%04x htb rate %s ceil %s",
          parentId, currentId, rate, ceil);

        if (burst != NULL)
          printf (" burst %s", burst);
      
        if (prio != NULL)
          printf (" prio %s", prio);

        printf (" %s\n", HTB_QUANTUM);
      }
      else
        printf ("# class name: %s, id: 0x01:0x%04x for input interface, skipped (requested)\n",
          name, currentId);

      if (htbOutif == NULL)
        goto classContinue;

      if ((useIf & XML_HTB_IFOUT) > 0) {

        printf ("$TC class add dev $OUTDEV parent 0x01:0x%04x classid 0x01:0x%04x htb rate %s ceil %s",
          parentId, currentId, outrate, outceil);

        if (burst != NULL)
          printf (" burst %s", burst);
      
        if (prio != NULL)
          printf (" prio %s", prio);

        printf (" %s\n", HTB_QUANTUM);
      }
      else
        printf ("# class name: %s, id: 0x01:0x%04x for output interface, skipped (requested)\n",
          name, currentId);

classContinue:

      xmlFree (rate);
      xmlFree (ceil);
      xmlFree (outrate);
      xmlFree (outceil);
      xmlFree (name);
      xmlFree (burst);
      xmlFree (prio);
      xmlFree (onlyif);

    }

    /*

      Processing <filter> element

    */

    if (! xmlStrcmp (cur->name, "filter")) {

      uint32_t tmpVal;

      tmpVal = 0;
      
      if (depth > 0)
        lvlStackGet (depth - 1, &tmpVal);

      parentId = (depth - 1) * CLS_PER_DEPTH + tmpVal - 1;

      /* get filter properties */

      type = xmlGetProp (cur, "type");
      src = xmlGetProp (cur, "src");
      dst = xmlGetProp (cur, "dst");
      sport = xmlGetProp (cur, "sport");
      dport = xmlGetProp (cur, "dport");
      name = xmlGetProp (cur, "name");
      mark = xmlGetProp (cur, "mark");
      prio = xmlGetProp (cur, "prio");

      onlyif = xmlGetProp (cur->parent, "onlyif");

      if ((strcmp (type, "u32") == 0) && (src == NULL) && (dst == NULL) && (sport == NULL) && (dport == NULL)) {
        printf ("# %s filter skipped: src/dst nor sport/dport is specified\n", type);
        goto filterContinueMissing;
      }

      if ((strcmp (type, "fw") == 0) && (mark == NULL)) {
        printf ("# %s filter skipped: it's fw but no mark value specified\n", type);
      }

      if (name != NULL) {
        printf ("# %s\n", name);
      }

      useIf = XML_HTB_IFALL;

      if (onlyif != NULL) {
        if (! xmlStrcmp (onlyif, "in")) {
          useIf &= XML_HTB_IFIN;
          goto filterIfCheckDone;
        }

        if (! xmlStrcmp (onlyif, "out"))
          useIf &= XML_HTB_IFOUT;
      }

filterIfCheckDone:

      /* u32 filter */

      if (strcmp (type, "u32") == 0) {

        if ((useIf & XML_HTB_IFIN) > 0) {

          printf ("$TC filter add dev $DEV parent 0x01:0x00 prio %s protocol ip u32 \\\n", prio);
          printf ("   ");
        
          if (src != NULL)
            printf (" match ip src %s", src);

          if (dst != NULL)
            printf (" match ip dst %s", dst);

          if (sport != NULL)
            printf (" match ip sport %s 0xffff ",sport);

          if (dport != NULL)
            printf (" match ip dport %s 0xffff ",dport);

          printf (" classid 0x01:0x%04x\n", parentId);

        }
        else
          printf ("# %s filter, for class 0x01:0x%04x on input, skipped (requested)\n",
            type, parentId);

        if (htbOutif == NULL)
          goto filterContinue1;

        if ((useIf & XML_HTB_IFOUT) > 0) {

          printf ("$TC filter add dev $OUTDEV parent 0x01:0x00 prio %s protocol ip u32 \\\n", prio);
          printf ("   ");
        
          if (src != NULL)
            printf (" match ip dst %s", src);
        
          if (dst != NULL)
            printf (" match ip src %s", dst);

          if (sport != NULL)
            printf (" match ip dport %s 0xffff ",sport);
        
          if (dport != NULL)
            printf (" match ip sport %s 0xffff ",dport);

          printf (" classid 0x01:0x%04x\n", parentId);
        }
        else
          printf ("# %s filter, for class 0x01:0x%04x on output, skipped (requested)\n",
            type, parentId);

      }

filterContinue1:

      /* fw filter */

      if (strcmp (type, "fw") == 0) {

        if ((useIf & XML_HTB_IFIN) > 0) {

          printf ("$TC filter add dev $DEV parent 0x01:0x00 prio %s protocol ip \\\n", prio);
          printf ("    handle %s fw classid 0x01:0x%04x\n", mark, parentId);

        }
        else
          printf ("# %s filter, for class 0x01:0x%04x on output, skipped (requested)\n",
            type, parentId);

      }

filterContinueMissing:

      xmlFree (type);
      xmlFree (src);
      xmlFree (dst);
      xmlFree (sport);
      xmlFree (dport);
      xmlFree (name);
      xmlFree (mark);
      xmlFree (prio);
      xmlFree (onlyif);

    }

    /*

      Processing <leaf> element

    */

    if (! xmlStrcmp (cur->name, "leaf")) {

      uint32_t tmpVal;

      tmpVal = 0;
      
      if (depth > 0)
        lvlStackGet (depth - 1, &tmpVal);

      parentId = (depth - 1) * CLS_PER_DEPTH + tmpVal - 1;

      onlyif = xmlGetProp (cur->parent, "onlyif");
      data = xmlGetProp (cur, "data");

      useIf = XML_HTB_IFALL;

      if (data != NULL)
        leafData = data;
      else
        leafData = htbClassLeaf;

      if (onlyif != NULL) {
        if (! xmlStrcmp (onlyif, "in")) {
          useIf &= XML_HTB_IFIN;
          goto leafIfCheckDone;
        }
         
        if (! xmlStrcmp (onlyif, "out"))
          useIf &= XML_HTB_IFOUT;
      }

leafIfCheckDone:

      if ((useIf & XML_HTB_IFIN) > 0) {

        printf ("$TC qdisc add dev $DEV parent 0x01:0x%04x handle 0x%04x:0x00 %s\n",
          parentId, parentId, leafData);

      }
      else
        printf ("# leaf for class id: 0x01:0x%04x on input interface, skipped (requested)\n",
          parentId);

      if (htbOutif == NULL)
        goto leafContinue;

      if ((useIf & XML_HTB_IFOUT) > 0) {

        printf ("$TC qdisc add dev $OUTDEV parent 0x01:0x%04x handle 0x%04x:0x00 %s\n",
          parentId, parentId, htbClassLeaf);

      }
      else
        printf ("# leaf for class id: 0x01:0x%04x on output interface, skipped (requested)\n",
          parentId);

leafContinue:

      xmlFree (onlyif);
      xmlFree (data);

      /*        
        Added printf to format well the script :)
      */
      printf ("\n");
    }

    lvlIdx++;
  } /* end if element node */

  if (cur->children != NULL) {
    depth++;
    lvlStackSet (depth - 1, lvlIdx);
    lvlStackGet (depth, &lvlIdx);

    handleDoc (doc, cur->children);

    lvlStackSet (depth, lvlIdx);
    lvlStackGet (depth - 1, &lvlIdx);
    depth--;
  }

  if (cur->next != NULL) {
    handleDoc (doc, cur->next);
  }
}
